<?php

namespace Ep\Images\Middleware;

use Ep\Images\Image;
use Ep\Images\ImageManager;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Zend\Diactoros\Response;

/**
 * Class Manager
 *
 * @package Ep\Images\Middleware
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class Manager implements MiddlewareInterface
{
    /** @var ImageManager */
    protected $images;

    /**
     * ImageManagerMiddleware constructor.
     *
     * @param ImageManager $images
     */
    public function __construct(ImageManager $images)
    {
        $this->images = $images;
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     *
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $method = $request->getMethod();

        switch ($method) {
            case 'DELETE':
                $response = $this->delete($request);
                break;
            case 'PUT':
            case 'POST':
                $response = $this->post($request);
                break;
            default:
                $response = $this->get($request);
        }

        return $response;
    }

    protected function get(ServerRequestInterface $request)
    {
        $path = $request->getUri()->getPath();
        $path = ltrim($path, '/');
        $pieces = explode('/', $path);

        // <namespace>/<cropped>?/<size>/<path>/<to>/</img>
        $namespace = array_shift($pieces);
        $cropped = array_shift($pieces);
        if ($cropped !== 'cropped') {
            array_unshift($pieces, $cropped);
            $cropped = null;
        }
        $size = array_shift($pieces);

        $imageName = $namespace . '/' . implode('/', $pieces);

        // <path>/<to>/<namespace>/<cropped>?/<size>/<image>
        /*
                $rpath = $request->getServerParams()['REQUEST_URI'];
                $rpath = ltrim($rpath, '/');

                $rpieces = explode('/', $rpath);
                $rpieces = array_reverse($rpieces);
                $imageName = array_shift($rpieces);

                $size = array_shift($rpieces);
                $cropped = array_shift($rpieces);
                if ($cropped !== 'cropped') {
                    array_unshift($rpieces, $cropped);
                    $cropped = null;
                }

                $rpieces = array_reverse($rpieces);
                $imageName = implode('/', $rpieces) . '/' . $imageName;
        */

        if (!preg_match('/^\d+x\d+$/', $size)) {
            return new Response\JsonResponse('Bad request');
        }

        list($width, $height) = explode('x', $size);

        try {
            $image = $this->images->getImage($imageName);
        } catch (\Exception $e) {
            return (new Response\JsonResponse(
                $e->getMessage(),
                404
            ));
        }

        $image->resize($width, $height, (bool)$cropped);

//        return new JsonResponse(['url' => $resizedPath]);
        return (new Response\HtmlResponse(
            $image->getData()
        ))->withHeader('Content-Type', $image->getType());
    }

    protected function post(ServerRequestInterface $request)
    {
        $rpath = $request->getServerParams()['REQUEST_URI'];
        $rpath = ltrim($rpath, '/');

        $image = json_decode($request->getBody()->getContents(), true);

//        $image['path'] = 'autokreso/test^&nest/what s up dawgg_+>.png';
//        $image['data'] = base64_encode(file_get_contents('images/0w20.png'));

        if ($image === null) {
            throw new \RuntimeException(sprintf(
                "Došlo je do greške prilikom dekodiranja JSON-a: %s",
                json_last_error_msg()
            ));
        }


        $encodedImagePath = implode('/', array_map([$this, 'customEncode'], explode('/', $image['path'])));

        $imagePath = $this->getFullPath($rpath . '/' . $encodedImagePath, '/');
        $imageDir = dirname($imagePath);


        if (!is_dir($imageDir)) {
            mkdir($imageDir, 0755, true);
        }

        if (preg_match('#^https?://#', $image['path'], $matches)) {
            $imageData = file_get_contents($image['path']);
        } else {
            $imageData = base64_decode($image['data']);
        }

        $image = new Image();
        $image->setPath($imagePath);
        $image->setData($imageData);

        return new Response\JsonResponse([
            'path' => $imagePath
        ]);
    }

    protected function put(ServerRequestInterface $request)
    {

    }

    protected function delete(ServerRequestInterface $request)
    {
        $rpath = $request->getServerParams()['REQUEST_URI'];
        $rpath = ltrim($rpath, '/');

        $image = json_decode($request->getBody()->getContents(), true);

//        $image['path'] = 'bla/tet.png';
//        $image['data'] = base64_encode(file_get_contents('images/0w20.png'));


        if ($image === null) {
            throw new \RuntimeException(sprintf(
                "Došlo je do greške prilikom dekodiranja JSON-a: %s",
                json_last_error_msg()
            ));
        }

        $encodedImagePath = implode('/', array_map([$this, 'customEncode'], explode('/', $image['path'])));

        $path = $rpath . ltrim($encodedImagePath, '/');
        $paths = glob(sprintf('%s/{,*/,*/*/}%s', $this->images->getBasePath(), $path), GLOB_BRACE);

        foreach ($paths as $p) {
            unlink($p);
        }

        return new Response\JsonResponse('');
    }

    /**
     * @param string $imagePath
     * @return string
     */
    protected function getFullPath(string $imagePath)
    {
        if (preg_match('#^https?://[^/]+/(.+)$#', $imagePath, $matches)) {
            $imagePath = $matches[1];
        }

        return sprintf(
            '%s/%s',
            $this->images->getBasePath(),
            ltrim(preg_replace('/\\\\/', '/', $imagePath), '/')
        );
    }

    /**
     * Rawurlencode + decode reserved characters
     *
     * @param string $str
     * @return mixed
     */
    protected function customEncode(string $str)
    {
        $reservedCharacters = [
            '%21' => '!',
            '%24' => '$',
            '%26' => '&',
            '%27' => '\'',
            '%28' => '(',
            '%29' => ')',
            '%2A' => '*',
            '%2B' => '+',
            '%2C' => ',',
            '%2D' => '-',
            '%2E' => '.',
//            '%2F' => '/',
            '%3A' => ':',
            '%3B' => ';',
            '%3D' => '=',
            '%40' => '@',
        ];

        return str_replace(array_keys($reservedCharacters), array_values($reservedCharacters), rawurlencode($str));
    }


}