<?php

namespace Ep\Images\Sprite;

/**
 * Class SpriteMap
 *
 * @package Ep
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-core
 */
class SpriteMap
{
    const IMAGES_PER_ROW = 1000;

    /** @var string */
    protected $name;
    /** @var string */
    protected $base_path;
    /** @var string */
    protected $output_path;
    /** @var SpriteImage[] */
    protected $images = [];
    /** @var bool */
    protected $dirty = false;
    /** @var array */
    protected $sizes = [];

    /**
     * SpriteMap constructor.
     *
     * @param string $name
     * @param array $config
     */
    public function __construct(string $name, array $config)
    {
        $this->name = $name;
        $this->base_path = $config['base_path'];
        $this->output_path = $this->base_path . '/sprites';

        if (!is_dir($this->output_path)) {
            mkdir($this->output_path, 0777, true);
        }

//        $this->output_path = realpath($this->output_path);

        $jsonPath = sprintf('%s/%s.json', $this->output_path, $this->name);

        if (file_exists($jsonPath)) {
            $this->sizes = json_decode((string)file_get_contents($jsonPath), true);
        }
    }

    public function getData(int $width, int $height)
    {
        $filename = sprintf('%s/%s-%dx%d.png',
            $this->output_path,
            $this->name,
            $width,
            $height
        );

        return file_get_contents($filename);
    }

    /**
     * Renderira sliku u sprajtu.
     *
     * @param string $imgName Ime slike
     * @param int $width Sirina
     * @param int $height Visina
     *
     * @return string
     * @throws \ImagickException
     */
    public function render($imgName, int $width, int $height)
    {
        $size = sprintf('%dx%d', $width, $height);

        if (!isset($this->sizes[$size])) {
            $this->sizes[$size] = [
                'name' => [],
                'map' => [],
            ];
        }

        if (!isset($this->sizes[$size]['name'][$imgName])) {
            $this->sizes[$size]['name'][$imgName] = $this->add($imgName, $width, $height);
        }

        [$indexX, $indexY] = $this->sizes[$size]['name'][$imgName];

        $bgLeft = $indexX * $width;
        $bgTop = $indexY * $height;

        return sprintf(
            '%d %d %d',
            $bgLeft,
            $bgTop,
            $indexX
        );
    }


    /**
     * Sprema promjene.
     * @throws \ImagickException
     */
    public function flush()
    {
        if ($this->dirty) {
            foreach ($this->images as $image) {
                $image->flush();
            }

            $jsonPath = sprintf('%s/%s.json', $this->output_path, $this->name);
            $json = json_encode($this->sizes);

            file_put_contents($jsonPath, $json);
        }
    }

    /**
     * Dodaje sliku u sprite i vraca koordinate.
     *
     * @param string $imgName
     * @param int $width
     * @param int $height
     *
     * @return array
     * @throws \ImagickException
     */
    protected function add(string $imgName, int $width, int $height)
    {
        $image = $this->getImage($width, $height);
        $size = sprintf('%dx%d', $width, $height);
        $imgPath = $this->base_path . '/' . $imgName;

        // sliku dodajemo u zadnji red
        $indexY = !empty($this->sizes[$size]['map']) ? count($this->sizes[$size]['map']) - 1 : 0;
        // ako red ne postoji, napravimo ga
        if (!isset($this->sizes[$size]['map'][$indexY])) {
            $this->sizes[$size]['map'][$indexY] = [];
        }

        // ako vec imamo dovoljno slika u ovom redu, pravimo novi red
        if (count($this->sizes[$size]['map'][$indexY]) === self::IMAGES_PER_ROW) {
            $indexY++;
            $this->sizes[$size]['map'][$indexY] = [];
        }

        $indexX = count($this->sizes[$size]['map'][$indexY]);

        $image->add($imgPath, $width, $height, $indexX, $indexY);
        $this->sizes[$size]['map'][$indexY][$indexX] = $imgName;
        $this->dirty = true;

        return [$indexX, $indexY];
    }

    /**
     * @param int $width
     * @param int $height
     * @return string
     */
    public function getImageUrl(int $width, int $height)
    {
        return sprintf(
            '%s/%s-%dx%d.png',
            $this->output_path,
            $this->name,
            $width,
            $height
        );
    }

    /**
     * Vraca sliku za date dimenzije.
     *
     * @param int $width
     * @param int $height
     *
     * @return SpriteImage
     */
    protected function getImage(int $width, int $height)
    {
        $fileName = sprintf('%s-%dx%d.png', $this->name, $width, $height);

        if (!isset($this->images[$fileName])) {
            $filePath = sprintf('%s/%s', $this->output_path, $fileName);

            $image = new SpriteImage($this->base_path, $filePath, $width, $height);

            $this->images[$fileName] = $image;
        }

        return $this->images[$fileName];
    }
}
